import { enumToEnumOptionMapper, getI18nKeyPrefix, getTranslator, mapStateToControlProps, rankWith, } from '@jsonforms/core';
import { html, repeat } from '@microsoft/fast-element';
import { capitalCase } from 'change-case';
import { isOneOfEnum } from '../testers/isOneOfEnum';
import { ENUM_CONTROL_RANK } from './RenderersRanks';
// overrides https://github.com/eclipsesource/jsonforms/blob/69441cdfc48f359850d87fa4b842856f1f139eb2/packages/core/src/util/renderer.ts#L515
// to allow mapping of enums that come as oneOf
export const mapStateToEnumControlProps = (state, ownProps) => {
    const props = mapStateToControlProps(state, ownProps);
    const options = ownProps.options ||
        (props.schema.enum || props.schema.oneOf.find((x) => x.hasOwnProperty('enum')).enum).map((e) => enumToEnumOptionMapper(e, getTranslator()(state), getI18nKeyPrefix(props.schema, props.uischema, props.path))) ||
        (props.schema.const && [
            enumToEnumOptionMapper(props.schema.const, getTranslator()(state), getI18nKeyPrefix(props.schema, props.uischema, props.path)),
        ]);
    return Object.assign(Object.assign({}, props), { options });
};
const updateEnumData = (dispatchRenderer, data) => {
    var _a;
    dispatchRenderer.control.handleChange(dispatchRenderer.control.path, (_a = dispatchRenderer.control.options.find((option) => capitalCase(option.label) === data)) === null || _a === void 0 ? void 0 : _a.value);
    dispatchRenderer.onBlur();
};
const displaySetOrDefaultData = (dispatchRenderer) => {
    var _a, _b, _c, _d;
    let data = dispatchRenderer.control.data;
    if (!data) {
        const enumDefault = (_c = (_b = (_a = dispatchRenderer.control.schema) === null || _a === void 0 ? void 0 : _a.oneOf) === null || _b === void 0 ? void 0 : _b.find((x) => x.hasOwnProperty('default'))) === null || _c === void 0 ? void 0 : _c.default;
        if (enumDefault) {
            data = enumDefault;
            updateEnumData(dispatchRenderer, data);
        }
    }
    return ((_d = dispatchRenderer.control.options.find((option) => option.value === data)) === null || _d === void 0 ? void 0 : _d.label) || '';
};
export const EnumControlRendererTemplate = (prefix = 'zero') => html `
  <template>
    <control-wrapper
      :control=${(x) => x.control}
      :jsonForms=${(x) => x.jsonforms}
      :touched=${(x) => x.touched}
      ?submitted=${(x) => x.submitted}
      prefix="${prefix}"
    >
      <${prefix}-combobox
        style="width: 100%"
        autocomplete="both"
        ?disabled=${(x) => !x.control.enabled}
        :value=${(x) => displaySetOrDefaultData(x)}
        @change=${(x, c) => updateEnumData(x, c.event.target.value)}
        id=${(x) => x.control.path}
        data-test-id=${(x) => x.control.path}
      >
        ${repeat((x) => x.control.options, html `
            <${prefix}-option>${(x) => capitalCase(x.label)}</${prefix}-option>
          `)}
      </${prefix}-combobox>
    </control-wrapper>
  </template>
`;
export const EnumControlRendererEntry = {
    renderer: html `
    ${(x) => EnumControlRendererTemplate(x.prefix)}
  `,
    tester: rankWith(ENUM_CONTROL_RANK, isOneOfEnum),
    mapper: mapStateToEnumControlProps,
};
